/**
 * <tt>Base64</tt> fornisce le funzioni di codifica/decodifica di un buffer in/da BASE64 secondo le specifiche definite
 * nella RFC-1521.
 *
 * @author  Simone Perina
 */
 
package examples;
 
public class Base64
{  
   //definisce l'alfabeto di codifica   
   private static final byte TO_BASE64[] = {
      65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 
      75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 
      85, 86, 87, 88, 89, 90, 97, 98, 99, 100, 
      101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 
      111, 112, 113, 114, 115, 116, 117, 118, 119, 120,
      121, 122, 48, 49, 50, 51, 52, 53, 54, 55, 
      56, 57, 43, 47 };
   
   //definisce l'alfabeto di decodifica   
   private static final byte FROM_BASE64[] = {
      -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 
      -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 
      -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 
      -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 
      -1, -1, -1, 62, -1, -1, -1, 63, 52, 53, 
      54, 55, 56, 57, 58, 59, 60, 61, -1, -1, 
      -1, -1, -1, -1, -1, 0, 1, 2, 3, 4, 
      5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 
      15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 
      25, -1, -1, -1, -1, -1, -1, 26, 27, 28, 
      29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 
      39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 
      49, 50, 51 };
      
   /**
    * Codifica un buffer in BASE64.
    *
    * @param      buffer e' il buffer con i dati da codificare
    * @return     il buffer con i dati codificati
    */      
   public static final byte[] encode(byte[] buffer)
   {
      return encode(buffer, 0, buffer.length);           
   }
      
   /**
    * Codifica un buffer in BASE64.
    *
    * @param      buffer e' il buffer con i dati da codificare
    * @param      offset e' l'offset del buffer da codificare
    * @param      length e' la lunghezza del buffer da codificare
    * @return     il buffer con i dati codificati
    */        
   public static final byte[] encode(byte[] buffer, int offset, int length)
   {        
      int size, i, j;
      int b1, b2, b3;   
      
      //calcola la lunghezza del buffer con i dati codificati
      int group = length / 3;
      int rest = length % 3;      
      if (rest != 0) size = (group + 1) * 4; else size = group * 4;            
      byte[] decoded = new byte[size];
      
      //codifica gruppi di 3 bytes alla volta           
      for (i = offset, j = 0; group > 0; group--)
      {
         b1 = buffer[i] < 0 ? buffer[i++] + 256 : buffer[i++];
         b2 = buffer[i] < 0 ? buffer[i++] + 256 : buffer[i++];
         b3 = buffer[i] < 0 ? buffer[i++] + 256 : buffer[i++];
         decoded[j++] = TO_BASE64[b1 >> 2 & 0x3f];
         decoded[j++] = TO_BASE64[(b1 << 4 | b2 >> 4) & 0x3f];
         decoded[j++] = TO_BASE64[(b2 << 2 | b3 >> 6) & 0x3f];
         decoded[j++] = TO_BASE64[b3 & 0x3f];         
      }
      
      //considera anche i bytes rimanenti   
      if (rest != 0)
      {
         b1 = buffer[i] < 0 ? buffer[i++] + 256 : buffer[i++];
         decoded[j++] = TO_BASE64[b1 >> 2];
         if (rest == 1)
         {
            decoded[j++] = TO_BASE64[b1 << 4 & 0x3f];
            decoded[j++] = 61;
            decoded[j++] = 61;
         } 
         else
         {
            b2 = buffer[i] < 0 ? buffer[i++] + 256 : buffer[i++];
            decoded[j++] = TO_BASE64[(b1 << 4 | b2 >> 4) & 0x3f ];
            decoded[j++] = TO_BASE64[b2 << 2 & 0x3f];
            decoded[j++] = 61;
         }
      }
                  
      return decoded;
   }
   
   /**
    * Decodifica una stringa in BASE64.
    *
    * @param      buffer e' il buffer con i dati da decodificare
    * @return     il buffer con i dati decodificati
    */   
   public static final byte[] decode(byte[] buffer)
   {
      return decode(buffer, 0, buffer.length);           
   }   
      
   /**
    * Decodifica una stringa in BASE64.
    *
    * @param      buffer e' il buffer con i dati da decodificare
    * @param      offset e' l'offset del buffer da decodificare
    * @param      length e' la lunghezza del buffer da decodificare
    * @return     il buffer con i dati decodificati
    */         
   public static final byte[] decode(byte[] buffer, int offset, int length)
   {
      //controlla che la lunghezza del buffer da decodificare sia corretta
      if (length % 4 != 0) throw new IllegalArgumentException("Invalid buffer length.");
            
      int size, i, j;
      int b1, b2, b3, b4;
      
      //calcola la lunghezza del buffer con i dati decodificati
      int group = length / 4;    
      int rest = 0;            
      if (length > 1 && buffer[length - 2] == 61)
      {
         rest = 2;
      }      
      else   
      {
         if (length > 0 && buffer[length - 1] == 61) rest = 1;
      }  
      size = group * 3 - rest;         
      byte[] encoded = new byte[size];
      
      //decodifica gruppi di 4 bytes alla volta                       
      for (i = offset, j = 0; group > 0; group--)
      {
         b1 = FROM_BASE64[buffer[i++]];
         b2 = FROM_BASE64[buffer[i++]];
         b3 = FROM_BASE64[buffer[i++]];
         b4 = FROM_BASE64[buffer[i++]];
         encoded[j++] = (byte)((b1 << 2 | b2 >> 4) & 0xff);
         encoded[j++] = (byte)((b2 << 4 | b3 >> 2) & 0xff);
         encoded[j++] = (byte)((b3 << 6 | b4) & 0xff);
      }
      
      //considera anche i bytes rimanenti      
      if (rest != 0)
      {
         b1 = FROM_BASE64[buffer[i++]];
         b2 = FROM_BASE64[buffer[i++]];
         encoded[j++] = (byte)((b1 << 2 | b2 >> 4) & 0xff);
         if (rest == 1)
         {
            b3 = FROM_BASE64[buffer[i++]];
            encoded[j++] = (byte)((b2 << 4 | b3 >> 2) & 0xff);
         }
      }
            
      return encoded;
   }      
}