/*
 * L'applet DataApplet e' l'applicazione client usata per comunicare con la servlet DataServlet.
 *
 * DataApplet crea e invia come richiesta un oggetto serializzato con i parametri di input per DataServlet quindi riceve 
 * come risposta un altro oggetto serializzato che visualizza come tabella.
 *
 * Ambiente di sviluppo: Jakarta Tomcat 5.0.27 e Sun JDK 1.4.2
 *
 * @author  Simone Perina
 */
 
package examples.applet2servlet; 

import java.io.*;
import java.net.*;
import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;

import examples.*;

public class DataApplet extends JApplet implements ActionListener
{
   //definisce l'URL della servlet
   private static final String SERVLET_URL = "http://localhost:8080/test/tunnelingHTTP/DataServlet";
   
   //definisce utente e password di accesso alla servlet
   private static final String SERVLET_USER = "tomcat";      
   private static final String SERVLET_PASSWORD = "tomcat";    
   
   //definisce i limite per la generazione casuale dei parametri di input della servlet
   private static final int MIN_BASE = 1;  
   private static final int MAX_BASE = 1000;  
   private static final int MIN_ROWS = 3;  
   private static final int MAX_ROWS = 5;  
   private static final int MIN_COLUMNS = 3;  
   private static final int MAX_COLUMNS = 5;     
   
   private JLabel status;
   private JButton update;   
   private DefaultTableModel model;  
   
   /**
    * Inizializza l'applet.
    */   
   public void init() 
   {
      String message;
      
      try
      {
         //imposta i parametri di input della servlet
         DataInput input = new DataInput();
         input.setBase((int) (Math.random() * (MAX_BASE - MIN_BASE) + MIN_BASE));
         input.setRows((int) (Math.random() * (MAX_ROWS - MIN_ROWS) + MIN_ROWS));
         input.setColumns((int) (Math.random() * (MAX_COLUMNS - MIN_COLUMNS) + MIN_COLUMNS));
         
         //invia la richiesta e riceve la risposta dalla servlet
         DataOutput output = manageDataServlet(SERVLET_URL, SERVLET_USER, SERVLET_PASSWORD, input);
         model = new DefaultTableModel(output.getItems(), output.getNames());      
         message = "ok";
      }
      catch (DataException e)
      {
         model = new DefaultTableModel();      
         message = e.getMessage();
      }
      
      //imposta la GUI dell'applet
      Container contentPane = getContentPane();
      JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
      status = new JLabel();
      status.setText(message);
      JTable table = new JTable();
      table.setModel(model);
      update = new JButton("Aggiorna");
      update.addActionListener(this);
      buttonPanel.add(update);
      contentPane.add(status, BorderLayout.NORTH);
      contentPane.add(new JScrollPane(table), BorderLayout.CENTER);
      contentPane.add(buttonPanel, BorderLayout.SOUTH);
   }
   
   /**
    * Avvia l'applet.
    */   
   public void start()
   {
      repaint();
   }
   
   /**
    * Gestore dell'evento bottone premuto.
    * 
    * @param actionEvent e' l'evento associato al bottone premuto
    */   
  	public void actionPerformed(ActionEvent actionEvent)
	{
	   String message;
	   
	   //se il bottone premuto e' "aggiorna" allora ricarica i dati dell'applicazione
		if (actionEvent.getSource() == update)
		{		
         try
         {  
            DataInput input = new DataInput();
            input.setBase((int) (Math.random() * (MAX_BASE - MIN_BASE) + MIN_BASE));
            input.setRows((int) (Math.random() * (MAX_ROWS - MIN_ROWS) + MIN_ROWS));
            input.setColumns((int) (Math.random() * (MAX_COLUMNS - MIN_COLUMNS) + MIN_COLUMNS));
            
            //invia la richiesta e riceve la risposta dalla servlet
            DataOutput output = manageDataServlet(SERVLET_URL, SERVLET_USER, SERVLET_PASSWORD, input);
            model.setDataVector(output.getItems(), output.getNames());
            message = "ok";
         }
         catch (DataException e)
         {
            message = e.getMessage();
         }
         
         status.setText(message);			           
		}
	}   
	
   /**
    * Gestisce il colloquio con la servlet DataServlet.
    *
    * @param      servlet e' l'URL di DataServlet
    * @param      user e' l'utente di accesso a DataServlet
    * @param      password e' la password di accesso a DataServlet
    * @param      input e' l'input di DataServlet
    * @return     l'output di DataServlet
    * @exception  <tt>DataException</tt> in caso di errore
    */	
	private DataOutput manageDataServlet(String servlet, String user, String password, DataInput input) throws DataException
	{	   
	   ObjectOutputStream outputStream = null;
	   ObjectInputStream inputStream = null;	   	      
	   	   	   	   
	   try
	   {
	      //crea l'URL e la connessione HTTP con la servlet
   	   URL url = new URL(servlet);    	    	   
         HttpURLConnection httpURLConnection = (HttpURLConnection)url.openConnection();
         
         //abilita l'input, l'output e disabilita la cache 
         httpURLConnection.setDoInput(true);
         httpURLConnection.setDoOutput(true);
         httpURLConnection.setUseCaches(false);      
         
         //imposta l'utente e la password di accesso alla servlet protetta da Basic Authentication
         String userPassword = user + ":" + password;      	   
			httpURLConnection.setRequestProperty("Authorization", "Basic " + new String(Base64.encode(userPassword.getBytes())));						            
         
         //invia i parametri di input alla servlet come oggetto serializzato
         httpURLConnection.setRequestProperty("Content-Type", "application/x-java-serialized-object");
         outputStream = new ObjectOutputStream(httpURLConnection.getOutputStream());
         outputStream.writeObject(input);         
         
         //riceve i parametri di output della servlet come oggetto serializzato         
         inputStream = new ObjectInputStream(httpURLConnection.getInputStream());
         DataOutput output = (DataOutput) inputStream.readObject();         
         
         return output;
      }
      catch(MalformedURLException e)
      {
         throw new DataException("MalformedURLException=" + e.getMessage());
	   }      
      catch(IOException e)
      {
         throw new DataException("IOException=" + e.getMessage());
	   }
      catch(ClassNotFoundException e)
      {
         throw new DataException("ClassNotFoundException=" + e.getMessage());
	   }            	  
	   finally
      {
	      try
	      {         
            if (outputStream != null) outputStream.close();
            if (inputStream != null) inputStream.close();                  
         }
         catch(IOException e)
         {
   	   }         
	   }	   	    	   
   }
}