/*
 * L'applet StringApplet e' l'applicazione client usata per comunicare con la servlet StringServlet.
 *
 * StringApplet crea e invia come richiesta una stringa di lunghezza casuale quindi riceve e visualizza come risposta 
 * un'altra stringa di lunghezza variabile.
 *
 * Ambiente di sviluppo: Jakarta Tomcat 5.0.27 e Sun JDK 1.4.2
 *
 * @author  Simone Perina
 */
 
package examples.applet2servlet; 

import java.io.*;
import java.net.*;
import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import examples.*;

public class StringApplet extends JApplet implements ActionListener
{
   //definisce l'URL della servlet
   private static final String SERVLET_URL = "http://localhost:8080/test/tunnelingHTTP/StringServlet";
   
   //definisce utente e password di accesso alla servlet
   private static final String SERVLET_USER = "tomcat";      
   private static final String SERVLET_PASSWORD = "tomcat";    
   
   //definisce i limiti per la generazione della stringa di lunghezza casuale
   private static final int MIN_STRING_LENGTH = 100;  
   private static final int MAX_STRING_LENGTH = 1000;  
   
   private JTextArea area;
   private JLabel status;
   private JButton update;   
   
   /**
    * Inizializza l'applet.
    */   
   public void init() 
   {
      String output;
      String message;
      
      try
      {
         //imposta i parametri di input della servlet generando una stringa di lunghezza casuale
         int length =  (int) (Math.random() * (MAX_STRING_LENGTH  - MIN_STRING_LENGTH) + MIN_STRING_LENGTH);
         StringBuffer stringBuffer = new StringBuffer();
         while (stringBuffer.length() < length) stringBuffer.append("input\n");
         System.out.println("DEBUG: input=" + stringBuffer.length() + " bytes");
         
         //invia la richiesta e riceve la risposta dalla servlet
         output = manageStringServlet(SERVLET_URL, SERVLET_USER, SERVLET_PASSWORD, stringBuffer.toString());
         message = "ok";
      }
      catch (DataException e)
      {
         output = "";
         message = e.getMessage();
      }
      
      //imposta la GUI dell'applet
      Container contentPane = getContentPane();
      JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
      status = new JLabel();
      status.setText(message);
      area = new JTextArea(output);
      update = new JButton("Aggiorna");
      update.addActionListener(this);
      buttonPanel.add(update);
      contentPane.add(status, BorderLayout.NORTH);
      contentPane.add(new JScrollPane(area), BorderLayout.CENTER);
      contentPane.add(buttonPanel, BorderLayout.SOUTH);
   }
   
   /**
    * Avvia l'applet.
    */   
   public void start()
   {
      repaint();
   }
   
   /**
    * Gestore dell'evento bottone premuto.
    * 
    * @param actionEvent e' l'evento associato al bottone premuto
    */   
  	public void actionPerformed(ActionEvent actionEvent)
	{
	   String output;
	   String message;
	   
	   //se il bottone premuto e' "aggiorna" allora ricarica i dati dell'applicazione
		if (actionEvent.getSource() == update)
		{		
         try
         {  
            //imposta i parametri di input della servlet generando una stringa di lunghezza casuale
            int length =  (int) (Math.random() * (MAX_STRING_LENGTH  - MIN_STRING_LENGTH) + MIN_STRING_LENGTH);
            StringBuffer stringBuffer = new StringBuffer();
            while (stringBuffer.length() < length) stringBuffer.append("input\n");            
            System.out.println("DEBUG: input=" + stringBuffer.length() + " bytes");
            
            //invia la richiesta e riceve la risposta dalla servlet
            output = manageStringServlet(SERVLET_URL, SERVLET_USER, SERVLET_PASSWORD, stringBuffer.toString());
            area.setText(output);
            message = "ok";
         }
         catch (DataException e)
         {
            output = "";
            message = e.getMessage();
         }
         
         status.setText(message);			           
		}
	}   
	
   /**
    * Gestisce il colloquio con la servlet StringServlet.
    *
    * @param      servlet e' l'URL di StringServlet
    * @param      user e' l'utente di accesso a StringServlet
    * @param      password e' la password di accesso a StringServlet
    * @param      input e' l'input di StringServlet
    * @return     l'output di StringServlet
    * @exception  <tt>DataException</tt> in caso di errore
    */	
	private String manageStringServlet(String servlet, String user, String password, String input) throws DataException
	{	   
	   DataOutputStream outputStream = null;
	   DataInputStream inputStream = null;	   	      
	   	   	   	   
	   try
	   {
	      //crea l'URL e la connessione HTTP con la servlet
   	   URL url = new URL(servlet);    	    	   
         HttpURLConnection httpURLConnection = (HttpURLConnection)url.openConnection();
         
         //abilita l'input, l'output e disabilita la cache 
         httpURLConnection.setDoInput(true);
         httpURLConnection.setDoOutput(true);
         httpURLConnection.setUseCaches(false);      
         
         //imposta l'utente e la password di accesso alla servlet protetta da Basic Authentication
         String userPassword = user + ":" + password;      	   
			httpURLConnection.setRequestProperty("Authorization", "Basic " + new String(Base64.encode(userPassword.getBytes())));						            
         
         //invia i parametri di input della servlet come stringa; il contenuto MIME "application/octet-stream" indica 
         //un flusso di bytes
         httpURLConnection.setRequestProperty("Content-Type", "application/octet-stream ");
         outputStream = new DataOutputStream(httpURLConnection.getOutputStream());
         outputStream.write(input.getBytes());    
         
         //riceve i parametri di output della servlet come stringa; l'intestazione "content-length" indica il numero di
         //bytes da leggere dal corpo del messaggio
         int contentLength = httpURLConnection.getContentLength();
         byte[] buffer = new byte[contentLength];        
         inputStream = new DataInputStream(httpURLConnection.getInputStream());
         int bytesRead = inputStream.read(buffer);   
         String output = new String(buffer);
         
         System.out.println("DEBUG: output=" + bytesRead + " bytes");
         
         return output;
      }
      catch(MalformedURLException e)
      {
         throw new DataException("MalformedURLException=" + e.getMessage());
	   }      
      catch(IOException e)
      {
         throw new DataException("IOException=" + e.getMessage());
	   }
	   finally
      {
	      try
	      {         
            if (outputStream != null) outputStream.close();
            if (inputStream != null) inputStream.close();                  
         }
         catch(IOException e)
         {
   	   }         
	   }	   	    	   
   }
}