/*
 * La servlet esegue il download di un contenuto multimediale da un sito protetto con certificato lato client.
 *
 * clientKeyStore.jks e' il keystore con la chiave privata e il certificato personale del client
 * clientTrustStore.jsp e' il truststore con il certificato radice del server
 *
 * Lo script buildKeys.bat fornisce un esempio di utilizzo di keytool per la generazione del keystore e del truststore.
 *
 * Ambiente di sviluppo: Jakarta Tomcat 5.0.27 e Sun JDK 1.4.2
 *
 * @author  Simone Perina
 */
 
package examples.servlet2cgi; 

import java.io.*;
import java.net.*;
import java.util.*;
import java.text.*;

import javax.servlet.*;
import javax.servlet.http.*;

import java.security.*;
import javax.net.ssl.*;

import examples.*;

public class DownloadServlet extends HttpServlet implements Servlet
{
	/**
	 * Esegue il download di un contenuto multimediale da un sito protetto servendo una richiesta con metodo GET.
	 *
	 * @param httpServletRequest e' la richiesta HTTP inviata dal client
	 * @param httpServletResponse e' la risposta HTTP da inviare al client
	 * @exception <tt>ServletException</tt> in caso di errore nella gestione della richiesta
	 * @exception <tt>IOException</tt> in caso di errore di input/output nella gestione della richiesta
	 */   
	public void doGet(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) throws ServletException, IOException 
	{			
      download(httpServletRequest, httpServletResponse);
	}
	
	/**
	 * Esegue il download di un contenuto multimediale da un sito protetto servendo una richiesta con metodo POST.
	 *
	 * @param httpServletRequest e' la richiesta HTTP inviata dal client
	 * @param httpServletResponse e' la risposta HTTP da inviare al client
	 * @exception <tt>ServletException</tt> in caso di errore nella gestione della richiesta
	 * @exception <tt>IOException</tt> in caso di errore di input/output nella gestione della richiesta
	 */	
	public void doPost(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) throws ServletException, IOException 
	{			
      download(httpServletRequest, httpServletResponse);
	}	
	
	/**
	 * Esegue il download di un contenuto multimediale da sito un protetto.
	 *
	 * @param httpServletRequest e' la richiesta HTTP inviata dal client
	 * @param httpServletResponse e' la risposta HTTP da inviare al client
	 * @exception <tt>ServletException</tt> in caso di errore nella gestione della richiesta
	 * @exception <tt>IOException</tt> in caso di errore di input/output nella gestione della richiesta
	 */	
	private void download(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) throws ServletException, IOException 
	{				   	   	   
		SSLContext sslContext = null;
		BufferedInputStream inputStream = null;
		BufferedOutputStream outputStream = null;		
		
		//imposta l'URL del sito di download
		String site = "https://localhost:9043/cgi-bin/download.exe";
		
		//imposta il server proxy
		String proxyHost = "server";
		String proxyPort = "port";		
	   String proxyUser = "user";
		String proxyPassword = "password";
		String proxyUserPassword = proxyUser + ":" + proxyPassword;
		String proxyAuthorization = new String(Base64.encode(proxyUserPassword.getBytes()));						
		
		//imposta il gestore del protocollo usato da JSSE per gestire la connessione HTTPS	
		System.setProperty("java.protocol.handler.pkgs", "sun.net.www.protocol");
		
		//imposta il server proxy usato da JSSE usando le proprieta' di sistema 'https.proxyHost' e 'https.proxyPort'
      System.setProperty("https.proxyHost", proxyHost);
      System.setProperty("https.proxyPort", proxyPort);									
		
		//imposta il keystore con la chiave privata e il certificato del client
		String keyStoreFile = "d:/tunnelingHTTP/clientKeyStore.jks";
		String keyStorePassword = "keyPassword";		
		String keyPassword = "myKeyPassword";		
		
		//imposta il truststore con il certificato radice del server
		String trustStoreFile = "d:/tunnelingHTTP/clientTrustStore.jks";
		String trustStorePassword = "trustPassword"; 		

      //imposta i parametri CGI della richiesta										
		HashMap cgi = new HashMap();
		cgi.put("parameter1", httpServletRequest.getParameter("parameter1"));
		cgi.put("parameter2", httpServletRequest.getParameter("parameter2"));
						
		try
		{		                          
		   //carica dal keystore la chiave privata e il certificato del client
		   KeyStore keyStore = KeyStore.getInstance("JKS");
			keyStore.load(new FileInputStream(keyStoreFile), keyStorePassword.toCharArray());
			KeyManagerFactory keyManagerFactory = KeyManagerFactory.getInstance("SunX509");
			keyManagerFactory.init(keyStore, keyPassword.toCharArray());

         //carica dal truststore il certificato radice del server
			KeyStore trustStore = KeyStore.getInstance("JKS");
			trustStore.load(new FileInputStream(trustStoreFile), trustStorePassword.toCharArray());			   		  
			TrustManagerFactory trustManagerFactory = TrustManagerFactory.getInstance("SunX509");
			trustManagerFactory.init(trustStore );

         //crea il contesto SSL per l'autenticazione lato client
			sslContext = SSLContext.getInstance("SSLv3");
			sslContext.init(keyManagerFactory.getKeyManagers(), trustManagerFactory.getTrustManagers(), null);  
						
			//crea l'URL al sito di download
			URL urlSite = new URL(site);
						
			//se la fase di autenticazione ha avuto successo crea la connessione HTTPS
			HttpsURLConnection httpsURLConnection = (HttpsURLConnection) urlSite.openConnection();
			httpsURLConnection.setSSLSocketFactory(sslContext.getSocketFactory());
			
			//imposta la connessione per l'input, l'output e disabilita la cache 
			httpsURLConnection.setDoInput(true);
			httpsURLConnection.setDoOutput(true);
			httpsURLConnection.setUseCaches(false);
						
			//imposta utente e password di accesso del server proxy
			httpsURLConnection.setRequestProperty("Proxy-Authorization", "Basic " + proxyAuthorization);			
					
		   //imposta i parametri CGI con metodo POST della richiesta HTTP
			httpsURLConnection.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");			
			DataOutputStream dataOutputStream = new DataOutputStream(httpsURLConnection.getOutputStream());
			dataOutputStream.writeBytes(cgiToString(cgi));
			dataOutputStream.close();		
						
			//imposta l'intestazione della risposta della servlet
			httpServletResponse.setHeader("Content-Type", httpsURLConnection.getContentType());  	
			httpServletResponse.setHeader("Content-Length", String.valueOf(httpsURLConnection.getContentLength()));
			httpServletResponse.setHeader("Last-Modified", new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss z").format(new Date(httpsURLConnection.getLastModified())));
						
			//scarica il contenuto dal sito e lo scrive come risposta della servlet
			inputStream = new BufferedInputStream(httpsURLConnection.getInputStream());
			outputStream = new BufferedOutputStream(httpServletResponse.getOutputStream());						
			byte[] buffer = new byte[8192];
			for(;;) 
			{
				int bytesRead = inputStream.read(buffer);
				if (bytesRead < 0) break; 
				outputStream.write(buffer, 0, bytesRead);
			}										
		}     		
		catch(Exception e)
		{
		   System.out.println("Exception=" + e.getMessage());
		}      
		finally
		{
			if (inputStream != null) inputStream.close();
			if (outputStream != null) outputStream.close();			
		}	
	}	
	
	/**
	 * Converte i parametri CGI in una stringa codificata da usare nella richiesta HTTP.
	 *
	 * @param cgi e' l'elenco delle variabili CGI da codificare
	 * @return la stringa con le variabili CGI codificate
	 */	
	private String cgiToString(HashMap cgi)
	{
	   try
	   {
         StringBuffer stringBuffer= new StringBuffer();
         Iterator iterator = cgi.keySet().iterator();
         while (iterator.hasNext()) 
         {
            String name = (String) iterator.next();
            String value = (String) cgi.get(name);
            stringBuffer.append(URLEncoder.encode(name, "8859_1")).append('=').append(URLEncoder.encode(value, "8859_1"));
            if (iterator.hasNext()) stringBuffer.append('&');
         }
         return stringBuffer.toString();
      }
      catch(UnsupportedEncodingException e)
      {
         return null;
      }
	}			
}
